@@ -36,6 +36,11 @@ LOW
 
 #define DEFAULT_SERVER_PORT        7878
 
+namespace OstProto {
+    class PortContent;
+    class PortGroupContent;
+}
+
 class QFile;
 class QTimer;
 
@@ -62,11 +67,14 @@ private:
     OstProto::PortIdList       *portIdList_;
     OstProto::PortStatsList    *portStatsList_;
 
+    OstProto::PortGroupContent *atConnectConfig_;
+    QList<const OstProto::PortContent*> atConnectPortConfig_;
+
 public: // FIXME(HIGH): member access
     QList<Port*>        mPorts;
 
 public:
-    PortGroup(QHostAddress ip = QHostAddress::LocalHost, 
+    PortGroup(QString serverName = "127.0.0.1",
         quint16 port = DEFAULT_SERVER_PORT); 
     ~PortGroup();
 
@@ -75,23 +83,27 @@ public:
         compat = kUnknown;
         rpcChannel->establish(); 
     }
-    void connectToHost(QHostAddress ip, quint16 port) { 
+    void connectToHost(QString serverName, quint16 port) {
         reconnect = true;
         compat = kUnknown;
-        rpcChannel->establish(ip, port);
+        rpcChannel->establish(serverName, port);
     }
     void disconnectFromHost() { reconnect = false; rpcChannel->tearDown(); }
 
+    void setConfigAtConnect(const OstProto::PortGroupContent *config);
+
     int numPorts() const { return mPorts.size(); }
+    int numReservedPorts() const;
     quint32 id() const { return mPortGroupId; } 
 
     const QString& userAlias() const { return mUserAlias; } 
     void setUserAlias(QString alias) { mUserAlias = alias; };
 
-    const QHostAddress& serverAddress() const 
-        { return rpcChannel->serverAddress(); } 
+    const QString serverName() const
+        { return rpcChannel->serverName(); }
     quint16 serverPort() const 
         { return rpcChannel->serverPort(); } 
+    const QString serverFullName() const;
     QAbstractSocket::SocketState state() const {
         if (compat == kIncompatible)
             return QAbstractSocket::SocketState(-1);
@@ -106,17 +118,31 @@ public:
     void processDeleteStreamAck(PbRpcController *controller);
     void processModifyStreamAck(int portIndex, PbRpcController *controller);
 
+    void processAddDeviceGroupAck(PbRpcController *controller);
+    void processDeleteDeviceGroupAck(PbRpcController *controller);
+    void processModifyDeviceGroupAck(int portIndex, PbRpcController *controller);
+
+    void processDeviceList(int portIndex, PbRpcController *controller);
+    void processDeviceNeighbors(int portIndex, PbRpcController *controller);
+
     void modifyPort(int portId, OstProto::Port portConfig);
-    void processModifyPortAck(PbRpcController *controller);
+    void processModifyPortAck(bool restoreUi, PbRpcController *controller);
     void processUpdatedPortConfig(PbRpcController *controller);
 
     void getStreamIdList();
     void processStreamIdList(int portIndex, PbRpcController *controller);
-    void getStreamConfigList();
+    void getStreamConfigList(int portIndex);
     void processStreamConfigList(int portIndex, PbRpcController *controller);
 
     void processModifyStreamAck(OstProto::Ack *ack);
 
+    void getDeviceGroupIdList();
+    void processDeviceGroupIdList(int portIndex, PbRpcController *controller);
+    void getDeviceGroupConfigList(int portIndex);
+    void processDeviceGroupConfigList(
+            int portIndex,
+            PbRpcController *controller);
+
     void startTx(QList<uint> *portList = NULL);
     void processStartTxAck(PbRpcController *controller);
     void stopTx(QList<uint> *portList = NULL);
@@ -129,6 +155,11 @@ public:
     void viewCapture(QList<uint> *portList = NULL);
     void processViewCaptureAck(PbRpcController *controller);
 
+    void resolveDeviceNeighbors(QList<uint> *portList = NULL);
+    void processResolveDeviceNeighborsAck(PbRpcController *controller);
+    void clearDeviceNeighbors(QList<uint> *portList = NULL);
+    void processClearDeviceNeighborsAck(PbRpcController *controller);
+
     void getPortStats();
     void processPortStatsList();
     void clearPortStats(QList<uint> *portList = NULL);
@@ -154,6 +185,7 @@ private slots:
 
 public slots:
     void when_configApply(int portIndex);
+    void getDeviceInfo(int portIndex);
 
 };
 
