@@ -19,18 +19,24 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "portswindow.h"
 
-#include "abstractfileformat.h"
+#include "deviceswidget.h"
 #include "portconfigdialog.h"
 #include "settings.h"
 #include "streamconfigdialog.h"
+#include "streamfileformat.h"
 #include "streamlistdelegate.h"
 
+#include "fileformat.pb.h"
+
 #include <QFileInfo>
 #include <QInputDialog>
 #include <QItemSelectionModel>
+#include <QMainWindow>
 #include <QMessageBox>
 #include <QSortFilterProxyModel>
 
+extern QMainWindow *mainWindow;
+
 PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     : QWidget(parent), proxyPortModel(NULL)
 {
@@ -44,6 +50,7 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     plm = pgl;
 
     setupUi(this);
+    devicesWidget->setPortGroupList(plm);
 
     tvPortList->header()->hide();
 
@@ -61,7 +68,7 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     tvPortList->addAction(actionExclusive_Control);
     tvPortList->addAction(actionPort_Configuration);
 
-    // Populate StramList Context Menu Actions
+    // Populate StreamList Context Menu Actions
     tvStreamList->addAction(actionNew_Stream);
     tvStreamList->addAction(actionEdit_Stream);
     tvStreamList->addAction(actionDuplicate_Stream);
@@ -74,12 +81,17 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     tvStreamList->addAction(actionOpen_Streams);
     tvStreamList->addAction(actionSave_Streams);
 
-    // PortList and StreamList actions combined make this window's actions
+    // PortList, StreamList, DeviceWidget actions combined
+    // make this window's actions
     addActions(tvPortList->actions());
     sep = new QAction(this);
     sep->setSeparator(true);
     addAction(sep);
     addActions(tvStreamList->actions());
+    sep = new QAction(this);
+    sep->setSeparator(true);
+    addAction(sep);
+    addActions(devicesWidget->actions());
 
     tvStreamList->setModel(plm->getStreamModel());
 
@@ -112,6 +124,9 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
         SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), 
         this, SLOT(when_portView_currentChanged(const QModelIndex&, 
             const QModelIndex&)));
+    connect(this,
+            SIGNAL(currentPortChanged(const QModelIndex&, const QModelIndex&)),
+            devicesWidget, SLOT(setCurrentPortIndex(const QModelIndex&)));
 
     connect(plm->getStreamModel(), SIGNAL(rowsInserted(QModelIndex, int, int)), 
         SLOT(updateStreamViewActions()));
@@ -128,7 +143,8 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     tvStreamList->resizeColumnToContents(StreamModel::StreamIcon);
     tvStreamList->resizeColumnToContents(StreamModel::StreamStatus);
 
-    // Initially we don't have any ports/streams - so send signal triggers
+    // Initially we don't have any ports/streams/devices
+    //  - so send signal triggers
     when_portView_currentChanged(QModelIndex(), QModelIndex());
     updateStreamViewActions();
 
@@ -157,6 +173,116 @@ PortsWindow::~PortsWindow()
     delete proxyPortModel;
 }
 
+int PortsWindow::portGroupCount()
+{
+    return plm->numPortGroups();
+}
+
+int PortsWindow::reservedPortCount()
+{
+    int count = 0;
+    int n = portGroupCount();
+
+    for (int i = 0; i < n; i++)
+        count += plm->portGroupByIndex(i).numReservedPorts();
+
+    return count;
+}
+
+//! Always return true
+bool PortsWindow::openSession(
+        const OstProto::SessionContent *session,
+        QString & /*error*/)
+{
+    QProgressDialog progress("Opening Session", NULL,
+                             0, session->port_groups_size(), mainWindow);
+    progress.show();
+    progress.setEnabled(true); // since parent (mainWindow) is disabled
+
+    plm->removeAllPortGroups();
+
+    for (int i = 0; i < session->port_groups_size(); i++) {
+        const OstProto::PortGroupContent &pgc = session->port_groups(i);
+        PortGroup *pg = new PortGroup(QString::fromStdString(
+                                                    pgc.server_name()),
+                                      quint16(pgc.server_port()));
+        pg->setConfigAtConnect(&pgc);
+        plm->addPortGroup(*pg);
+        progress.setValue(i+1);
+    }
+
+    return true;
+}
+
+/*!
+ * Prepare content to be saved for a session
+ *
+ * If port reservation is in use, saves only 'my' reserved ports
+ *
+ * Returns false, if user cancels op; true, otherwise
+ */
+bool PortsWindow::saveSession(
+        OstProto::SessionContent *session, // OUT param
+        QString & /*error*/,
+        QProgressDialog *progress)
+{
+    int n = portGroupCount();
+    QString myself;
+
+    if (progress) {
+        progress->setLabelText("Preparing Ports and PortGroups ...");
+        progress->setRange(0, n);
+    }
+
+    if (reservedPortCount())
+        myself = appSettings->value(kUserKey, kUserDefaultValue).toString();
+
+    for (int i = 0; i < n; i++)
+    {
+        PortGroup &pg = plm->portGroupByIndex(i);
+        OstProto::PortGroupContent *pgc = session->add_port_groups();
+
+        pgc->set_server_name(pg.serverName().toStdString());
+        pgc->set_server_port(pg.serverPort());
+
+        for (int j = 0; j < pg.numPorts(); j++)
+        {
+            if (myself != pg.mPorts.at(j)->userName())
+                continue;
+
+            OstProto::PortContent *pc = pgc->add_ports();
+            OstProto::Port *p = pc->mutable_port_config();
+
+            // XXX: We save the entire OstProto::Port even though some
+            // fields may be ephemeral; while opening we use only relevant
+            // fields
+            pg.mPorts.at(j)->protoDataCopyInto(p);
+
+            for (int k = 0; k < pg.mPorts.at(j)->numStreams(); k++)
+            {
+                OstProto::Stream *s = pc->add_streams();
+                pg.mPorts.at(j)->streamByIndex(k)->protoDataCopyInto(*s);
+            }
+
+            for (int k = 0; k < pg.mPorts.at(j)->numDeviceGroups(); k++)
+            {
+                OstProto::DeviceGroup *dg = pc->add_device_groups();
+                dg->CopyFrom(*(pg.mPorts.at(j)->deviceGroupByIndex(k)));
+            }
+        }
+
+        if (progress) {
+            if (progress->wasCanceled())
+                return false;
+            progress->setValue(i);
+        }
+        if (i % 2 == 0)
+            qApp->processEvents();
+    }
+
+    return true;
+}
+
 void PortsWindow::showMyReservedPortsOnly(bool enabled)
 {
     if (!proxyPortModel)
@@ -240,6 +366,8 @@ void PortsWindow::when_portView_currentChanged(const QModelIndex& currentIndex,
                     SLOT(updatePortRates()));
         }
     }
+
+    emit currentPortChanged(current, previous);
 }
 
 void PortsWindow::when_portModel_dataChanged(const QModelIndex& topLeft,
@@ -503,15 +631,29 @@ void PortsWindow::on_actionNew_Port_Group_triggered()
 {
     bool ok;
     QString text = QInputDialog::getText(this, 
-        "Add Port Group", "Port Group Address (IP[:Port])",
+        "Add Port Group", "Port Group Address (HostName[:Port])",
         QLineEdit::Normal, lastNewPortGroup, &ok);
     
     if (ok)
     {
         QStringList addr = text.split(":");
-        if (addr.size() == 1) // Port unspecified
-            addr.append(QString().setNum(DEFAULT_SERVER_PORT));
-        PortGroup *pg = new PortGroup(QHostAddress(addr[0]),addr[1].toUShort());    
+        quint16 port = DEFAULT_SERVER_PORT;
+
+        if (addr.size() > 2) { // IPv6 Address
+            // IPv6 addresses with port number SHOULD be specified as
+            // [2001:db8::1]:80 (RFC5952 Sec6) to avoid ambiguity due to ':'
+            addr = text.split("]:");
+            if (addr.size() > 1)
+                port = addr[1].toUShort();
+        }
+        else if (addr.size() == 2) // Hostname/IPv4 + Port specified
+            port = addr[1].toUShort();
+
+        // Play nice and remove square brackets irrespective of addr type
+        addr[0].remove(QChar('['));
+        addr[0].remove(QChar(']'));
+
+        PortGroup *pg = new PortGroup(addr[0], port);
         plm->addPortGroup(*pg);
         lastNewPortGroup = text;
     }
@@ -670,6 +812,9 @@ void PortsWindow::on_actionOpen_Streams_triggered()
 {
     qDebug("Open Streams Action");
 
+    QStringList fileTypes = StreamFileFormat::supportedFileTypes(
+                                            StreamFileFormat::kOpenFile);
+    QString fileType;
     QModelIndex current = tvPortList->selectionModel()->currentIndex();
     static QString dirName;
     QString fileName;
@@ -682,7 +827,11 @@ void PortsWindow::on_actionOpen_Streams_triggered()
 
     Q_ASSERT(plm->isPort(current));
 
-    fileName = QFileDialog::getOpenFileName(this, tr("Open Streams"), dirName);
+    if (fileTypes.size())
+        fileType = fileTypes.at(0);
+
+    fileName = QFileDialog::getOpenFileName(this, tr("Open Streams"),
+            dirName, fileTypes.join(";;"), &fileType);
     if (fileName.isEmpty())
         goto _exit;
 
@@ -736,7 +885,8 @@ void PortsWindow::on_actionSave_Streams_triggered()
 
     QModelIndex current = tvPortList->selectionModel()->currentIndex();
     static QString fileName;
-    QStringList fileTypes = AbstractFileFormat::supportedFileTypes();
+    QStringList fileTypes = StreamFileFormat::supportedFileTypes(
+                                            StreamFileFormat::kSaveFile);
     QString fileType;
     QString errorStr;
     QFileDialog::Options options;
@@ -745,9 +895,8 @@ void PortsWindow::on_actionSave_Streams_triggered()
         current = proxyPortModel->mapToSource(current);
 
     // On Mac OS with Native Dialog, getSaveFileName() ignores fileType 
-    // which we need.On some Linux distros the native dialog can't 
-    // distinguish between Ostinato(*) and PCAP(*)
-#if defined(Q_OS_MAC) || defined(Q_OS_UNIX)
+    // which we need
+#if defined(Q_OS_MAC)
     options |= QFileDialog::DontUseNativeDialog;
 #endif
 
@@ -762,6 +911,22 @@ _retry:
     if (fileName.isEmpty())
         goto _exit;
 
+    if (QFileInfo(fileName).suffix().isEmpty()) {
+        QString fileExt = fileType.section(QRegExp("[\\*\\)]"), 1, 1);
+        qDebug("Adding extension '%s' to '%s'",
+                qPrintable(fileExt), qPrintable(fileName));
+        fileName.append(fileExt);
+        if (QFileInfo(fileName).exists()) {
+            if (QMessageBox::warning(this, tr("Overwrite File?"), 
+                QString("The file \"%1\" already exists.\n\n"
+                    "Do you wish to overwrite it?")
+                    .arg(QFileInfo(fileName).fileName()),
+                QMessageBox::Yes|QMessageBox::No,
+                QMessageBox::No) != QMessageBox::Yes)
+                goto _retry;
+        }
+    }
+
     fileType = fileType.remove(QRegExp("\\(.*\\)")).trimmed();
     if (!fileType.startsWith("Ostinato") 
             && !fileType.startsWith("Python"))
