@@ -25,15 +25,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "ui_portswindow.h"
 #include "portgrouplist.h"
 
-/* TODO
-HIGH
-MED
-LOW
-*/
-
 class QAbstractItemDelegate;
+class QProgressDialog;
 class QSortFilterProxyModel;
 
+namespace OstProto {
+    class SessionContent;
+}
+
 class PortsWindow : public QWidget, private Ui::PortsWindow
 {
     Q_OBJECT
@@ -45,6 +44,19 @@ public:
     PortsWindow(PortGroupList *pgl, QWidget *parent = 0);
     ~PortsWindow();
 
+    int portGroupCount();
+    int reservedPortCount();
+
+    bool openSession(const OstProto::SessionContent *session,
+                     QString &error);
+    bool saveSession(OstProto::SessionContent *session,
+                     QString &error,
+                     QProgressDialog *progress = NULL);
+
+signals:
+    void currentPortChanged(const QModelIndex &current,
+                            const QModelIndex &previous);
+
 private:
     QString        lastNewPortGroup;
     QAbstractItemDelegate *delegate;
