@@ -24,8 +24,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "protocolmanager.h"
 
 extern ProtocolManager *OstProtocolManager;
+extern quint64 getDeviceMacAddress(int portId, int streamId, int frameIndex);
+extern quint64 getNeighborMacAddress(int portId, int streamId, int frameIndex);
 
-StreamBase::StreamBase() :
+StreamBase::StreamBase(int portId) :
+    portId_(portId),
     mStreamId(new OstProto::StreamId),
     mCore(new OstProto::StreamCore),
     mControl(new OstProto::StreamControl)
@@ -444,6 +447,27 @@ _exit:
     return true;
 }
 
+int StreamBase::frameSizeVariableCount() const
+{
+    int count = 1;
+
+    switch(lenMode())
+    {
+        case OstProto::StreamCore::e_fl_fixed:
+            break;
+        case OstProto::StreamCore::e_fl_inc:
+        case OstProto::StreamCore::e_fl_dec:
+        case OstProto::StreamCore::e_fl_random:
+            count = frameLenMax() - frameLenMin() + 1;
+            break;
+        default:
+            qWarning("%s: Unhandled len mode %d",  __FUNCTION__, lenMode());
+            break;
+    }
+
+    return count;
+}
+
 int StreamBase::frameVariableCount() const
 {
     ProtocolListIterator    *iter;
@@ -466,7 +490,7 @@ int StreamBase::frameVariableCount() const
     }
     delete iter;
 
-    return frameCount;
+    return AbstractProtocol::lcm(frameCount, frameSizeVariableCount());
 }
 
 // frameProtocolLength() returns the sum of all the individual protocol sizes
@@ -501,18 +525,22 @@ int StreamBase::frameCount() const
     return count;
 }
 
+// Returns packet length - if bufMaxSize < frameLen(), returns truncated
+// length i.e. bufMaxSize
 int StreamBase::frameValue(uchar *buf, int bufMaxSize, int frameIndex) const
 {
-    int        pktLen, len = 0;
+    int maxSize, size, pktLen, len = 0;
 
     pktLen = frameLen(frameIndex);
 
     // pktLen is adjusted for CRC/FCS which will be added by the NIC
     pktLen -= kFcsSize;
 
-    if ((pktLen < 0) || (pktLen > bufMaxSize))
+    if (pktLen <= 0)
         return 0;
 
+    maxSize = qMin(pktLen, bufMaxSize);
+
     ProtocolListIterator    *iter;
 
     iter = createProtocolListIterator();
@@ -524,17 +552,33 @@ int StreamBase::frameValue(uchar *buf, int bufMaxSize, int frameIndex) const
         proto = iter->next();
         ba = proto->protocolFrameValue(frameIndex);
 
-        if (len + ba.size() < bufMaxSize)
-            memcpy(buf+len, ba.constData(), ba.size());
-        len += ba.size();
+        size = qMin(ba.size(), maxSize-len);
+        memcpy(buf+len, ba.constData(), size);
+        len += size;
+
+        if (len == maxSize)
+            break;
     }
     delete iter;
 
-    // Pad with zero, if required
-    if (len < pktLen)
-        memset(buf+len, 0, pktLen-len);
+    // Pad with zero, if required and if we have space
+    if (len < maxSize) {
+        size = maxSize-len;
+        memset(buf+len, 0, size);
+        len += size;
+    }
 
-    return pktLen;
+    return len;
+}
+
+quint64 StreamBase::deviceMacAddress(int frameIndex) const
+{
+    return getDeviceMacAddress(portId_, int(mStreamId->id()), frameIndex);
+}
+
+quint64 StreamBase::neighborMacAddress(int frameIndex) const
+{
+    return getNeighborMacAddress(portId_, int(mStreamId->id()), frameIndex);
 }
 
 bool StreamBase::preflightCheck(QString &result) const
@@ -550,13 +594,18 @@ bool StreamBase::preflightCheck(QString &result) const
                 "frame length should be at least %1.\n")
                 .arg(frameProtocolLength(i) + kFcsSize);
             pass = false;
+            break;
         }
+    }
 
+    for (int i = 0; i < count; i++)
+    {
         if (frameLen(i) > 1522)
         {
             result += QString("Jumbo frames may be truncated or dropped "
                 "if not supported by the hardware\n");
             pass = false;
+            break;
         }
     }
 
