@@ -38,6 +38,8 @@ public:
     virtual bool hasExclusiveControl() { return false; }
     virtual bool setExclusiveControl(bool /*exclusive*/) { return false; }
 
+    virtual bool setRateAccuracy(AbstractPort::Accuracy accuracy); 
+
     virtual void clearPacketList() { 
         transmitter_->clearPacketList();
         setPacketListLoopMode(false, 0, 0);
@@ -68,6 +70,10 @@ public:
     virtual bool isCaptureOn()  { return capturer_->isRunning(); }
     virtual QIODevice* captureData() { return capturer_->captureFile(); }
 
+    virtual void startDeviceEmulation();
+    virtual void stopDeviceEmulation();
+    virtual int sendEmulationPacket(PacketBuffer *pktBuf);
+
 protected:
     enum Direction
     {
@@ -102,6 +108,9 @@ protected:
     public:
         PortTransmitter(const char *device);
         ~PortTransmitter();
+
+        bool setRateAccuracy(AbstractPort::Accuracy accuracy); 
+
         void clearPacketList();
         void loopNextPacketSet(qint64 size, qint64 repeats, 
             long repeatDelaySec, long repeatDelayNsec);
@@ -172,11 +181,10 @@ protected:
             long usecDelay_;
         };
 
-        void udelay(long usec);
+        static void udelay(unsigned long usec);
         int sendQueueTransmit(pcap_t *p, pcap_send_queue *queue, long &overHead,
                     int sync);
 
-        quint64 ticksFreq_;
         QList<PacketSequence*> packetSequenceList_;
         PacketSequence *currentPacketSequence_;
         int repeatSequenceStart_;
@@ -186,6 +194,8 @@ protected:
         int returnToQIdx_;
         quint64 loopDelay_;
 
+        void (*udelayFn_)(unsigned long);
+
         bool usingInternalStats_;
         AbstractPort::PortStats *stats_;
         bool usingInternalHandle_;
@@ -221,6 +231,32 @@ protected:
         volatile State  state_;
     };
 
+    class EmulationTransceiver: public QThread
+    {
+    public:
+        EmulationTransceiver(const char *device, DeviceManager *deviceManager);
+        ~EmulationTransceiver();
+        void run();
+        void start();
+        void stop();
+        bool isRunning();
+        int transmitPacket(PacketBuffer *pktBuf);
+
+    private:
+        enum State
+        {
+            kNotStarted,
+            kRunning,
+            kFinished
+        };
+
+        QString         device_;
+        DeviceManager   *deviceManager_;
+        volatile bool   stop_;
+        pcap_t          *handle_;
+        volatile State  state_;
+    };
+
     PortMonitor     *monitorRx_;
     PortMonitor     *monitorTx_;
 
@@ -229,6 +265,7 @@ protected:
 private:
     PortTransmitter *transmitter_;
     PortCapturer    *capturer_;
+    EmulationTransceiver *emulXcvr_;
 
     static pcap_if_t *deviceList_;
 };
